#ifndef __SHAPE_MODEL_H_
#define __SHAPE_MODEL_H_

#include <glm/glm.hpp>
#include <vector>


struct Vertex
{
  glm::vec3 position;
  glm::vec3 normal;
  glm::vec2 texcoord;
};

struct Material
{
  glm::vec4 ambient;
  glm::vec4 diffuse;
  glm::vec4 specular;
  float shininess;
};

struct SubData
{
  glm::uint materialIndex;
  glm::uint offset;
  glm::uint size;
};

struct Model
{
  Vertex               * vertices;
  glm::uint            * indices;

  std::vector<Material>  materials;
  std::vector<SubData>   subData;

  glm::uint numVertices;
  glm::uint numIndices;

  Model() :
    vertices(NULL),
    indices(NULL),
    materials(NULL),
    subData(NULL),
    numVertices(0),
    numIndices(0)
  {}
};


Model* createShape(
    Vertex*    vertices,
    glm::uint  numVertices,
    glm::uint* indices,
    glm::uint  numIndices
);
void addMaterial(Model* shape, const Material& material);
void destroyShape( Model* s );


#endif // __SHAPE_MODEL_H_
