//
// GLSL with GLEW - sample05
//
// $Id: sample05.cpp 41 2011-01-08 16:35:39Z bambooflow $
// $Rev: 41 $
// $Author: bambooflow $
//
#include <cstdlib>

#include "common.h"
#include "shape.h"


void reshape(int w, int h);
void init();
void timer(int t );
void display();



///////////////////////////////////////////////////////////////////////////////

Context context;

glm::mat4 viewMatrix;
glm::mat4 projectionMatrix;
 
Shape *sphere = NULL;
Shape *cube = NULL;
Light *light = NULL;


// Light
Material lightMaterial = {
  glm::vec4( 1.0f, 1.0f, 1.0f, 1.0f),
  glm::vec4( 1.0f, 1.0f, 1.0f, 1.0f),
  glm::vec4( 1.0f, 1.0f, 1.0f, 1.0f),
  1.0f
};

// Gold
Material material0 = {
  glm::vec4(0.247250, 0.199500, 0.074500, 1.0), // ambient
  glm::vec4(0.751640, 0.606480, 0.226480, 1.0), // diffuse
  glm::vec4(0.628281, 0.555802, 0.366065, 1.0), // specular
  51.2f                                         // shininess
};
 
// Blue
Material material1 = {
  glm::vec4(0.0, 0.0, 0.8f, 1.0), // ambient
  glm::vec4(0.8, 0.8, 0.8, 1.0),  // diffuse
  glm::vec4(0.8, 0.8, 0.8, 1.0),  // specular
  10.0f                           // shininess
};
 
void init()
{
  //
  // GL initialize
  //
  glClearColor(0.5, 0.5, 0.5, 1.0);
  glClearDepth(1.0f);

  viewMatrix = glm::lookAt(
      glm::vec3(3.0f, 4.0f, 5.0f), // eye pos
      glm::vec3(0.0f, 0.0f, 0.0f), // center pos
      glm::vec3(0.0f, 1.0f, 0.0f)  // up
  );

  glEnable(GL_DEPTH_TEST);
  glEnable(GL_CULL_FACE);


  //
  // GLSL initialize
  //
  context.programObj = createProgram("simple.vert", "simple.frag");

  context.modelLocation      = glGetUniformLocation(context.programObj, "u_modelMatrix");
  context.viewLocation       = glGetUniformLocation(context.programObj, "u_viewMatrix");
  context.projectionLocation = glGetUniformLocation(context.programObj, "u_projectionMatrix");

  // directional light param
  context.lightPositionLocation  = glGetUniformLocation(context.programObj, "u_lightPos");
  context.lightAmbientLocation   = glGetUniformLocation(context.programObj, "u_lightMaterial.ambient");
  context.lightDiffuseLocation   = glGetUniformLocation(context.programObj, "u_lightMaterial.diffuse");
  context.lightSpecularLocation  = glGetUniformLocation(context.programObj, "u_lightMaterial.specular");
  //context.lightShininessLocation = glGetUniformLocation(context.programObj, "u_lightMaterial.shininess");

  // material param
  context.materialAmbientLocation   = glGetUniformLocation(context.programObj, "u_material.ambient");
  context.materialDiffuseLocation   = glGetUniformLocation(context.programObj, "u_material.diffuse");
  context.materialSpecularLocation  = glGetUniformLocation(context.programObj, "u_material.specular");
  context.materialShininessLocation = glGetUniformLocation(context.programObj, "u_material.shininess");

  context.positionLocation   = glGetAttribLocation(context.programObj, "a_position");
  context.normalLocation     = glGetAttribLocation(context.programObj, "a_normal");

  glBindFragDataLocation(context.programObj, 0, "fragColor");

  //
  // setup light
  //
  light = new Light(&context);
  {
    Model* shape = createSphere(0.1f, 10);
    addMaterial( shape, lightMaterial );
    light->setup( shape );
  }
  light->setPosition(-2.0f, 1.0f, 0.0f); // direction
  light->setSize(1.0f, 1.0f, 1.0f);

  //
  // setup sphere
  //
  sphere = new Shape(&context);
  {
    Model* shape = createSphere(1.0f, 10);
    //Model* shape = loadModelFromObjFile("teapot.obj");
    addMaterial( shape, material0 );
    sphere->setup( shape );
  }
  sphere->setPosition(0.0f, 1.0f, 0.0f);
  sphere->setSize(0.5f, 0.5f, 0.5f);

  //
  // setup cube
  //
  cube = new Shape(&context);
  {
    Model* shape = createCube(1.0f);
    //Model* shape = loadModelFromObjFile("teapot.obj");
    addMaterial( shape, material1 );
    cube->setup( shape );
  }
  cube->setPosition(0.0f, -1.0f, 0.0f);
  cube->setSize(0.5f, 0.5f, 0.5f);
}

 
void reshape(int w, int h)
{
    glViewport(0, 0, w, h); 
 
    projectionMatrix = glm::perspective(30.0f, (float)w/(float)h, 1.0f, 100.0f);
}
 
void timer(int t)
{
    glutPostRedisplay();
    glutTimerFunc(t, timer, 17);
}
 

void display()
{
  static float angle = 0.0;

  light->setPosition( 2.0*sin(angle*M_PI/180.0f), 1.0f, 2.0*cos(angle*M_PI/180.0f) );
  //light->setPosition( 1.0, 2.0*sin(angle*M_PI/180.0f), 2.0*cos(angle*M_PI/180.0f) );
  light->update();
  sphere->update();
  cube->update();

  // ---------------------------------------------------------------------------

  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

  glUseProgram(context.programObj);
  {
    glUniformMatrix4fv(context.viewLocation, 1, GL_FALSE, glm::value_ptr(viewMatrix));
    glUniformMatrix4fv(context.projectionLocation, 1, GL_FALSE, glm::value_ptr(projectionMatrix));

    //
    // Draw Sphere
    //
    light->render();
    sphere->render();
    cube->render();

    // end
    glBindVertexArray(0);
    glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0);
  }
  glUseProgram(0);

  glutSwapBuffers();

  // ---------------------------------------------------------------------------

  angle += 1.0;
  if (angle >= 360.0f) angle = 0.0f;
}
 
 
int main(int argc, char *argv[])
{
  glutInit(&argc, argv);
  glutInitWindowSize(800, 600);
  glutInitDisplayMode(GLUT_RGBA | GLUT_DOUBLE | GLUT_DEPTH);
  glutCreateWindow("OpenGL with GLEW");

  glutDisplayFunc(display);
  glutReshapeFunc(reshape);
  glutTimerFunc(100, timer, 17);

  GLenum err;
  glewExperimental = GL_TRUE;
  err = glewInit();
  if (err != GLEW_OK) {
    std::cerr << "GLEW error : " << glewGetErrorString(err) << "\n";
    std::exit(1);
  }   
  if (!glewIsSupported("GL_VERSION_3_3")) {
    std::cerr << "OpenGL 3.3 not supported.\n";
    std::exit(1);
  }

  init();

  glUseProgram(context.programObj);

  glutMainLoop();

  return 0;
}

